#ifndef _AUDIOIO_CDR_IMPL_H
#define _AUDIOIO_CDR_IMPL_H

#define	BYTE	1
#define	WORD	2
#define	LONG	4
#define	FLOAT	5
#define DOUBLE	6

// #define SECTORSIZE	(2352 / 2)
#define SECTORSIZE	2352

#endif
